//**********************************************************************
//  DIVE Laboratories, Inc.
//  Copyright(c) 1995
//  All rights reserved
//  FILE:   MATERIAL.HPP
//
//  DESCRIPTION
//  This file provides the function prototypes for the materialClass
//
//  Author: M. Doucet
//
//  Modification History:
//  6/13/95	Created
//
//**********************************************************************
#ifndef _MATERIAL_HPP
#define _MATERIAL_HPP

#include "stack.hpp"
#include "vrtypes.hpp"

typedef enum {_AMBIENT, _DIFFUSE, _SPECULAR, _EMISSION} materialEnumType;
class materialClass {

   friend class geometryPrivateClass;
	friend class renderStateClass;
   friend class channelClass;
   friend class universeClass;

private:
	
	F3 ambient;
	F4 diffuse;
	F3 specular;
	F3 emission;

	float shininess;

	static stackClass stack;
	static F3 curAmbient;
	static F4 curDiffuse;
	static F3 curSpecular;
	static F3 curEmission;
	static float curShininess;

	char name[255];

	void  pushMaterial();
	void  popMaterial();
	void  makeCurrent();

public:

	static int stackIsEmpty;
	void  setComponentV(materialEnumType component, F4 vals);
	void  getComponentV(materialEnumType component, F4 vals);
	void  setComponentV(materialEnumType component, D4 vals);
	void  getComponentV(materialEnumType component, D4 vals);
	void  setShiny(float val);
	float getShiny();
	void  reset();

	void  setName(char *nm);
	char *getName();

	void operator=(materialClass *mat);

	materialClass();
	~materialClass();
};

#endif

